// coverage:ignore-file
import 'package:breaking_bad_quotes/core.dart';

/// Returns the [BreakingBadQuotesException] if any error occurs.
BreakingBadQuotesException getExceptionFromAny(dynamic error) {
  if (error is DefaultResult) {
    return getExceptionFromDefaultResult(error);
  }

  if (error is List) {
    if (error.isEmpty) {
      return UnknownException();
    } else if (error.length == 3) {
      return BreakingBadQuotesException(error[1] as String);
    }
  }

  return BreakingBadQuotesException('unknown exception: $error');
}

/// Returns the [BreakingBadQuotesException] by checking the [DefaultResult].
///
/// Like if any error happens due to invalid input to the request.
BreakingBadQuotesException getExceptionFromDefaultResult(DefaultResult result) {
  if (result.status == 400 && result.message == 'Invalid Arguments') {
    return InvalidArgumentsException();
  }

  if (result.status == 403 &&
      result.message!.startsWith('Use of the history Delete API')) {
    return MethodDisabledException(result.message!);
  }

  if (result.status == 403 && result.message == 'Forbidden') {
    return ForbiddenException(result.message!);
  }

  if (result.message?.isNotEmpty ?? false) {
    return BreakingBadQuotesException('Error: ${result.message}');
  }

  return BreakingBadQuotesException(Constants.unknownError);
}
