import 'package:breaking_bad_quotes/core.dart';

final _logger = injectLogger('breaking_bad_quotes.dx.quotes');

/// A mixin for Quotes which will get the quotes from the Breaking Bad.
mixin Quotes on Core {
  /// Get quotes
  ///
  /// Returns a list of Quotes from the Breaking Bad.
  ///
  /// If needed only 1 then no need to set [numberOfQuotes].
  /// The result will be always in List format even if [numberOfQuotes] is 1.
  ///
  /// For more details on the API go [here](https://github.com/shevabam/breaking-bad-quotes).
  ///
  /// Throws [BreakingBadQuotesException] on an error.
  Future<List<Results>> getQuotes({
    int numberOfQuotes = 1,
  }) async {
    var params = QuotesParams(
      numberOfQuotes: numberOfQuotes,
    );

    _logger.info('Getting $numberOfQuotes quotes from Breaking Bad');

    return defaultFlow<QuotesParams, List<Results>>(
      core: this,
      params: params,
      serializer: (dynamic json) => resultsFromMap(
        json as List<dynamic>,
      ),
    );
  }
}
