import 'package:breaking_bad_quotes/core.dart';
import 'package:test/test.dart';

/// Testing all the exceptions in the SDK
void main() {
  group(
    'testing exceptions',
    () {
      test(
        'will throw method disabled exception',
        () {
          expect(
            methodDisabledExceptionCall,
            throwsA(
              isA<MethodDisabledException>(),
            ),
          );

          MethodDisabledException? methodDisabledException;
          try {
            methodDisabledExceptionCall();
          } on MethodDisabledException catch (exception) {
            methodDisabledException = exception;
          }

          expect(methodDisabledException != null, true);
          expect(
            methodDisabledException!.message,
            equals(
              Constants.clientIsClosed,
            ),
          );
          expect(
            methodDisabledException.toString(),
            equals(
              Constants.clientIsClosed,
            ),
          );
        },
      );

      test(
        'will throw invalid argument exception',
        () {
          expect(
            invalidArgumentsExceptionCall,
            throwsA(
              isA<InvalidArgumentsException>(),
            ),
          );

          InvalidArgumentsException? anException;
          try {
            invalidArgumentsExceptionCall();
          } on InvalidArgumentsException catch (exception) {
            anException = exception;
          }

          expect(anException != null, true);
          expect(
            anException!.message,
            equals(
              Constants.invalidArgument,
            ),
          );
          expect(
            anException.toString(),
            equals(
              Constants.invalidArgument,
            ),
          );
        },
      );

      test(
        'will throw unknown exception',
        () {
          expect(
            unknownExceptionCall,
            throwsA(
              isA<UnknownException>(),
            ),
          );

          UnknownException? anException;
          try {
            unknownExceptionCall();
          } on UnknownException catch (exception) {
            anException = exception;
          }

          expect(anException != null, true);
          expect(
            anException!.message,
            equals(
              Constants.unknownError,
            ),
          );
          expect(
            anException.toString(),
            equals(
              Constants.unknownError,
            ),
          );
        },
      );

      test(
        'will throw forbidden exception',
        () {
          expect(
            forbiddenExceptionCall,
            throwsA(
              isA<ForbiddenException>(),
            ),
          );

          ForbiddenException? anException;
          try {
            forbiddenExceptionCall();
          } on ForbiddenException catch (exception) {
            anException = exception;
          }

          expect(anException != null, true);
          expect(
            anException!.message,
            equals(
              Constants.forbidden,
            ),
          );
          expect(
            anException.toString(),
            equals(
              Constants.forbidden,
            ),
          );
        },
      );

      test(
        'will throw request timeout exception',
        () {
          expect(
            requestTimeoutExceptionCall,
            throwsA(
              isA<RequestTimeoutException>(),
            ),
          );

          RequestTimeoutException? anException;
          try {
            requestTimeoutExceptionCall();
          } on RequestTimeoutException catch (exception) {
            anException = exception;
          }

          expect(anException != null, true);
          expect(
            anException!.message,
            equals(
              Constants.requestTimeout,
            ),
          );
          expect(
            anException.toString(),
            equals(
              Constants.requestTimeout,
            ),
          );
        },
      );

      test(
        'will throw request cancelled exception',
        () {
          expect(
            requestCancelExceptionCall,
            throwsA(
              isA<RequestCancelException>(),
            ),
          );

          RequestCancelException? anException;
          try {
            requestCancelExceptionCall();
          } on RequestCancelException catch (exception) {
            anException = exception;
          }

          expect(anException != null, true);
          expect(
            anException!.message,
            equals(
              Constants.requestCancelled,
            ),
          );
          expect(
            anException.toString(),
            equals(
              Constants.requestCancelled,
            ),
          );
        },
      );

      test(
        'will throw request other exception',
        () {
          expect(
            requestOtherExceptionCall,
            throwsA(
              isA<RequestOtherException>(),
            ),
          );

          RequestOtherException? anException;
          try {
            requestOtherExceptionCall();
          } on RequestOtherException catch (exception) {
            anException = exception;
          }

          expect(anException != null, true);
          expect(
            anException!.message,
            equals(
              '${Constants.requestFailed} (${Constants.requestFailed})',
            ),
          );
          expect(
            anException.toString(),
            equals(
              '${Constants.requestFailed} (${Constants.requestFailed})',
            ),
          );
        },
      );
    },
  );
}

void forbiddenExceptionCall() {
  throw ForbiddenException(
    Constants.forbidden,
  );
}

void invalidArgumentsExceptionCall() {
  throw InvalidArgumentsException();
}

void methodDisabledExceptionCall() {
  throw MethodDisabledException(
    Constants.clientIsClosed,
  );
}

void requestCancelExceptionCall() {
  throw RequestCancelException(
    Constants.requestCancelled,
  );
}

void requestOtherExceptionCall() {
  throw RequestOtherException(Constants.requestFailed);
}

void requestTimeoutExceptionCall() {
  throw RequestTimeoutException(
    Constants.requestTimeout,
  );
}

void unknownExceptionCall() {
  throw UnknownException();
}
